/*
 * BME280_OLED_Temp.ino
 * Sketch for BMP280 Temp / Pressure / Humidity sensor and 128x64 SSD1306 OLED
 * Uses I2C bus for sensor / display
 *
 * Copyright (c) 2018 Mark Mokoski.
 * Website    : www.k1pu.com
 * Author     : Mokoski, Mark
 * Created    : 17-JUL-2018
 * Create Time:
 * Change Log :
 *
 * The MIT License (MIT)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
 
#include <BME280I2C.h> // Libary is found at https://www.arduinolibraries.info/libraries/bme280
#include <Wire.h>
#include <Arduino.h>
#include <U8g2lib.h>

// Sensor constructor
BME280I2C bme;    // Default : forced mode, standby time = 1000 ms
                  // Oversampling = pressure ×1, temperature ×1, humidity ×1, filter off,

//Display constructor
U8G2_SSD1306_128X64_NONAME_1_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);

//U.S. Customary (=0) or Metric (=1) display select pin
int mode_pin = 12;

void setup()
{
  u8g2.begin();
  Wire.begin();
  bme.begin();
  
  pinMode(mode_pin, INPUT); // 1 = metric units, 0 = US customary units
  
/*  
  Serial.begin(9600); //set up sreial monitor
  while(!Serial) {} // Wait
  Serial.println("\n\n");//add a line at the begining.

  // Debug output for BME280 being present
  while(!bme.begin())  //test for BME280 being present and type
  {
    Serial.println("Could not find BME280 sensor!\n");
    delay(1000);
  }

  // bme.chipID(); // Deprecated. See chipModel().
  switch(bme.chipModel())
  {
     case BME280::ChipModel_BME280:
       Serial.println("Found BME280 sensor! Success.\n");
       break;
     case BME280::ChipModel_BMP280:
       Serial.println("Found BMP280 sensor! No Humidity available.\n");
       break;
     default:
       Serial.println("Found UNKNOWN sensor! Error!\n");
  }

*/

// Delay for 3 seconds for BME280 to settle out on power up before first reed
// Display the 'wait' message at start

 u8g2.firstPage();
    do {
      u8g2.setFont(u8g2_font_helvB14_tr);
      u8g2.drawStr(10,38,"Please Wait");
   
     } while ( u8g2.nextPage() );
    
  delay(3000); // 3 second start up wait for BME280 to settle out after power up

}

void loop()
{
  float temp_c(NAN), rel_hum(NAN), pres_Pa(NAN);
  float temp_f;
  float pres_inHg;
  float pres_hPa;
  float dewpoint_c;
  float dewpoint_f;
  float H; //temp value for dew point calc
  
  BME280::TempUnit tempUnit(BME280::TempUnit_Celsius);
  BME280::PresUnit presUnit(BME280::PresUnit_Pa);

  // Read ME280 data
  bme.read(pres_Pa, temp_c, rel_hum, tempUnit, presUnit);

  // Unit convertions

  temp_f = (temp_c * 9/5) + 32;            // Temp C to F
  pres_hPa = (pres_Pa/100);                // Pressure Pa to hPa
  pres_inHg = (pres_hPa* 0.02952998751);   // Pressure Pa to inHg

  /*
  // calculate Dew Point in deg C
  // this is an aproximation calc, tracks well with Rel Hum => 50%
  dewpoint_c = temp_c-((100 - rel_hum)/5);
  dewpoint_f = (dewpoint_c* 9/5) + 32;
  */
  
  // calculate Dew Point in deg C
  // this is a more acurate calc for dew point
  H = ((log10(rel_hum)-2)/0.4343) + ((17.62 * temp_c)/(243.12 + temp_c));
  dewpoint_c = (243.12 * H) / (17.62 - H);
  dewpoint_f = (dewpoint_c* 9/5) + 32;
  
  /*
  // Serial print for debug
  // print temperatures
  Serial.print("Temp: ");
  Serial.print(temp_c);
  Serial.println(" C"); // The unit for Celsius 
  
  Serial.print("Temp: ");
  Serial.print(temp_f);
  Serial.println(" F"); // The unit for Farenheit
  
  // print atmospheric pressure data
  Serial.print("Pressure: ");
  Serial.print(pres_hPa);
  Serial.println(" hPa"); //Unit for hecto Pascals

  Serial.print("Pressure: ");
  Serial.print(pres_inHg);
  Serial.println(" in. Hg"); // unit for inched of Mercury


  // print relative humidity data
  Serial.print("Relative Humidity: ");
  Serial.print(rel_hum);
  Serial.println("%");

  if (digitalRead(mode_pin) == 0) // 0 = U.S. Customary (F), 1 = Metric (C)
    {
      //print Display Unit data
      Serial.print("Display Units: ");
      Serial.println("U.S. Customary");
    }
  else
    {
      //print Display Unit data
      Serial.print("Display Units: ");
      Serial.println("Metric");
    }

  Serial.println("\n");//add a line between output of different times.      

  */
    
  //String var's for float convertion to display
  char str_temp_f[3];
  char str_temp_c[3];
  char str_pres_inHg[5];
  char str_pres_hPa[4];
  char str_rel_hum[4];
  char str_dp_f[3];
  char str_dp_c[3];

  if (digitalRead(mode_pin) == 0) // 0 = U.S. Customary (F), 1 = Metric (C)
    {
      // Display Temp in deg F and % Humidity - delay 20 seconds
    u8g2.firstPage();
      do {
        dtostrf(temp_f,3,0,str_temp_f); //convert float to 3 char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(13,28,str_temp_f);
        u8g2.setFont(u8g2_font_helvB12_tr);
        u8g2.drawStr(70,10,"o");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(80,28,"F");

        dtostrf(rel_hum,2,0,str_rel_hum); //convert float to nn.n char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_logisoso26_tr);
        u8g2.drawStr(1,60,str_rel_hum);
        u8g2.drawStr(40,60,"%");
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(68,44,"Relative");
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(68,60,"Humidity");
     
        } while ( u8g2.nextPage() );
    
    delay(20000); //display Temp / RH for 20 seconds

    //Disolay Dew Point in deg F - delay 10 seconds
    u8g2.firstPage();
      do {
        dtostrf(dewpoint_f,3,0,str_dp_f); //convert float to nn.nn char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(28,20,"Dew Point");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(13,58,str_dp_f);
        u8g2.setFont(u8g2_font_helvB12_tr);
        u8g2.drawStr(70,40,"o");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(80,58,"F");
        
      } while ( u8g2.nextPage() );
    
    delay(10000); //display Dew Point for 10 seconds


    //Disolay Barometric Pressure in in. Hg - delay 10 seconds
    u8g2.firstPage();
      do {
        dtostrf(pres_inHg,3,2,str_pres_inHg); //convert float to nn.nn char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(28,20,"Barometer");
        //u8g2.drawStr(32,27,"Pressure");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(1,58,str_pres_inHg);
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(90,53,"in Hg");
  
      } while ( u8g2.nextPage() );
    
    delay(10000); //display Barometric Pressure for 10 seconds

  }

  else //display Mertic Units if mode_pin = 1
    {
    // Display Temp in deg C - delay 20 seconds
    u8g2.firstPage();
      do {
        dtostrf(temp_c,3,0,str_temp_c); //convert float to 3 char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(13,28,str_temp_c);
        u8g2.setFont(u8g2_font_helvB12_tr);
        u8g2.drawStr(70,10,"o");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(80,28,"C");

        dtostrf(rel_hum,2,0,str_rel_hum); //convert float to nn.n char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_logisoso26_tr);
        u8g2.drawStr(1,60,str_rel_hum);
        u8g2.drawStr(40,60,"%");
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(68,44,"Relative");
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(68,60,"Humidity");
     
      } while ( u8g2.nextPage() );
    
    delay(20000); //display Temp / RH for 20 seconds

   //Disolay Dew Point in deg C - delay 10 seconds
    u8g2.firstPage();
      do {
        dtostrf(dewpoint_c,3,0,str_dp_c); //convert float to nn.nn char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(28,20,"Dew Point");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(13,58,str_dp_c);
        u8g2.setFont(u8g2_font_helvB12_tr);
        u8g2.drawStr(70,40,"o");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(80,58,"C");
        
      } while ( u8g2.nextPage() );
    
    delay(10000); //display Dew Point for 10 seconds

    //Disolay Barometric Pressure in in. Hg - delay 10 seconds
    u8g2.firstPage();
      do {
        dtostrf(pres_hPa,4,0,str_pres_hPa); //convert float to nn.nn char formatted string with rounding of the number
        u8g2.setFont(u8g2_font_helvB10_tr);
        u8g2.drawStr(28,20,"Barometer");
        u8g2.setFont(u8g2_font_logisoso28_tr);
        u8g2.drawStr(8,58,str_pres_hPa);
        u8g2.setFont(u8g2_font_helvB12_tr);
        u8g2.drawStr(88,51,"hPa");
  
      } while ( u8g2.nextPage() );
    
    delay(10000); //display Barometric Pressure for 10 seconds
    }
}
